/*requires Schema.Reporting.sql*/

DECLARE @ProcedureSchema NVARCHAR(256);
DECLARE @ProcedureName   NVARCHAR(256);

SET @ProcedureSchema = 'Reporting' ;
SET @ProcedureName   = 'ShredDeadlockHistoryTbl' ;

RAISERROR('-----------------------------------------------------------------------------------------------------------------',0,1);
RAISERROR('PROCEDURE [%s].[%s]',0,1,@ProcedureSchema,@ProcedureName);

IF  NOT EXISTS (SELECT 1 FROM sys.objects WHERE object_id = OBJECT_ID(N'[' + @ProcedureSchema + N'].[' + @ProcedureName +  N']') AND type in (N'P'))
BEGIN
    BEGIN TRY
        EXECUTE ('CREATE Procedure [' + @ProcedureSchema + '].[' + @ProcedureName +  '] ( ' +
                ' @ServerName    varchar(512), ' +
                ' @DbName    varchar(50) ' +
                ') ' +
                'AS ' +
                'BEGIN ' +
                '   SELECT ''Not implemented'' ' +
                'END')
    END TRY
    BEGIN CATCH
        PRINT '   Error while trying to create procedure'
        RETURN
    END CATCH

    PRINT '   PROCEDURE created.'
END
GO

ALTER PROCEDURE [Reporting].[ShredDeadlockHistoryTbl] (
    @SourceDatabaseName             VARCHAR(256)    = NULL,
    @SourceSchemaName               VARCHAR(256)    = NULL,
    @SourceTableName                VARCHAR(256),
    @DeadlockIdentifierColumnName   VARCHAR(256)    = 'DeadlockId',
    @DeadlockGraphColumnName        VARCHAR(256)    = 'DeadlockGraph',
    @TargetDatabaseName             VARCHAR(256)    = NULL,
    @TargetSchemaName               VARCHAR(256)    = NULL,
    @TargetTableName                VARCHAR(256),
    @Debug                          BIT             = 0
)
AS
/*
  ===================================================================================
    DESCRIPTION:

    PARAMETERS:

    REQUIREMENTS:

    EXAMPLE USAGE :
    
                EXEC [Reporting].[ShredDeadlockHistoryTbl] 
					@SourceSchemaName	= 'dbo',
					@SourceTableName	= 'DeadlockHistory',
					@TargetSchemaName   = 'dbo',
					@TargetTableName    = 'ShreddedDeadlockFromHistory',
					@Debug				= 1
  ===================================================================================
*/
BEGIN
    SET NOCOUNT ON;
    DECLARE @tsql                   nvarchar(max);
    DECLARE @LineFeed               CHAR(2);
    DECLARE @ExecRet                INT;
    DECLARE @TargetTableColumns     VARCHAR(MAX);
    DECLARE @SourceTableFullName    VARCHAR(1024);
    DECLARE @TargetTableFullName    VARCHAR(1024);
    DECLARE @logMsg                 VARCHAR(MAX);
    DECLARE @TmpCnt                 BIGINT;
    DECLARE @CurDeadlockId          BIGINT;
    DECLARE @ObjectLookupDbId       INT;
    DECLARE @ObjectLookupObjId      BIGINT;
    DECLARE @ObjectLookupName       VARCHAR(1024);
    
    SELECT
        @tsql                   = '',
        @LineFeed               = CHAR(13) + CHAR(10),
        @SourceDatabaseName     = ISNULL(@SourceDatabaseName,DB_NAME()),
        @SourceSchemaName       = ISNULL(@SourceSchemaName,SCHEMA_NAME()),
        @TargetDatabaseName     = ISNULL(@TargetDatabaseName,DB_NAME()),
        @TargetSchemaName       = ISNULL(@TargetSchemaName,SCHEMA_NAME()),
        @SourceTableFullName    = QUOTENAME(@SourceDatabaseName) + '.' + QUOTENAME(@SourceSchemaName) + '.' + QUOTENAME(@SourceTableName),
        @TargetTableFullName    = QUOTENAME(@TargetDatabaseName) + '.' + QUOTENAME(@TargetSchemaName) + '.' + QUOTENAME(@TargetTableName),
        @TargetTableColumns     = '    [DeadlockID]            BIGINT           NOT NULL,' + @LineFeed +
                                  '    [isVictim]              BIT              NOT NULL,' + @LineFeed +
                                  '    [ProcessQty]            INT              NOT NULL,' + @LineFeed +
                                  '    [ProcessNbr]            BIGINT           NOT NULL,' + @LineFeed +
                                  '    [LockMode]              VARCHAR(10)      NOT NULL,' + @LineFeed +
                                  '    [LockedObject]          VARCHAR(1024)    NULL,' + @LineFeed +
                                  '    [DatabaseId]            INT              NULL,' + @LineFeed +
                                  '    [AssociatedObjectId]    BIGINT           NULL,' + @LineFeed +
                                  '    [AssociatedObjectName]  VARCHAR(1024)    NULL,' + @LineFeed +
                                  '    [LockProcess]           VARCHAR(50)      NOT NULL,' + @LineFeed +
                                  '    [ThreadId]              INT              NOT NULL,' + @LineFeed +
                                  '    [SPID]                  INT              NOT NULL,' + @LineFeed +
                                  '    [SBID]                  INT              NOT NULL,' + @LineFeed +
                                  '    [ECID]                  INT              NOT NULL,' + @LineFeed +
                                  '    [TranCount]             INT              NOT NULL,' + @LineFeed +
                                  '    [LockEvent]             VARCHAR(8000)    NULL,' + @LineFeed +
                                  '    [LockedMode]            VARCHAR(10)      NULL,' + @LineFeed +
                                  '    [WaitProcessID]         VARCHAR(200)     NULL,' + @LineFeed +
                                  '    [WaitMode]              VARCHAR(10)      NULL,' + @LineFeed +
                                  '    [WaitResource]          VARCHAR(200)     NOT NULL,' + @LineFeed +
                                  '    [WaitType]              VARCHAR(100)     NULL,' + @LineFeed +
                                  '    [IsolationLevel]        VARCHAR(200)     NOT NULL,' + @LineFeed +
                                  '    [LogUsed]               BIGINT           NOT NULL,' + @LineFeed +
                                  '    [ClientApp]             VARCHAR(512)     NULL,' + @LineFeed +
                                  '    [HostName]              VARCHAR(512)     NOT NULL,' + @LineFeed +
                                  '    [LoginName]             VARCHAR(256)     NOT NULL,' + @LineFeed +
                                  '    [TransactionTime]       DATETIME         NOT NULL,' + @LineFeed +
                                  '    [BatchStarted]          DATETIME         NOT NULL,' + @LineFeed +
                                  '    [BatchCompleted]        DATETIME         NOT NULL,' + @LineFeed +
                                  '    [QueryStatement]        VARCHAR(max)     NULL,' + @LineFeed +
                                  '    [SQLHandle]             VARCHAR(64)      NULL,' + @LineFeed +
                                  '    [SQLHandleText]         VARCHAR(max)     NULL,' + @LineFeed +
                                  '    [InputBuffer]           XML              NULL,' + @LineFeed +
                                  '    [ExecutionStack]        XML              NULL'
                                  
    ;

    if (@Debug = 1)
    BEGIN
        RAISERROR('-- -----------------------------------------------------------------------------------------------------------------',0,1);
        RAISERROR('-- Now running [Reporting].[ShredDeadlockHistoryTbl] stored procedure.',0,1);
        RAISERROR('-- -----------------------------------------------------------------------------------------------------------------',0,1);
    END;

    EXEC @ExecRet = Utils.CheckTableExists 
                            @DatabaseName       = @SourceDatabaseName,
                            @SchemaName         = @SourceSchemaName,
                            @TableName          = @SourceTableName,
                            @RunCreateTable     = 0,
                            @RunTruncateTable   = 0,
                            @Debug              = @Debug
                    ;

    IF(@ExecRet <> 0)
    BEGIN
        RAISERROR('Source Table Name not found',12,1) WITH NOWAIT;
        RETURN;
    END;
    
    EXEC @ExecRet = Utils.CheckTableExists 
                            @DatabaseName       = @TargetDatabaseName,
                            @SchemaName         = @TargetSchemaName,
                            @TableName          = @TargetTableName,
                            @RunCreateTable     = 1,
                            @TableColumnsDef    = @TargetTableColumns,
                            @RunTruncateTable   = 0,
                            @Debug              = @Debug
                    ;               
                    
    IF(@ExecRet <> 0)
    BEGIN
        RAISERROR('Target Table Name not found or could not be created',12,1) WITH NOWAIT;
        RETURN;
    END;
    
    IF(OBJECT_ID('tempdb..#DeadlockOccurrences') IS NOT NULL)
    BEGIN
        exec sp_executesql N'DROP TABLE #DeadlockOccurrences';
    END;
    
    IF(OBJECT_ID('tempdb..#ObjectReferences') IS NOT NULL)
    BEGIN
        exec sp_executesql N'DROP TABLE #ObjectReferences';
    END;
    
    if(@Debug = 1)
    BEGIN
        RAISERROR('-- Existing temporary tables used in this procedure may have been dropped.',0,1) WITH NOWAIT;
    END;
    
    CREATE TABLE #DeadlockOccurrences (
        OccurId         BIGINT NOT NULL,
        InError         BIT DEFAULT 0,
        ErrorMessage    VARCHAR(MAX) NULL
    );
    
    CREATE TABLE #ObjectReferences (
        DatabaseId  INT NOT NULL,
        PartitionId BIGINT NOT NULL
    )
    
    SET @tsql = 'INSERT INTO #DeadlockOccurrences (OccurId)' + @LineFeed +
                'SELECT s.' + QUOTENAME(@DeadlockIdentifierColumnName) + '' + @LineFeed +
                'FROM ' + @SourceTableFullName + ' as s' + @LineFeed + 
                'EXCEPT' + @LineFeed +
                '    SELECT distinct DeadlockId FROM ' + @TargetTableFullName + @LineFeed
				;

    EXEC @ExecRet = sp_executesql @tsql ;
    
    IF(@Debug = 1)
    BEGIN
        SELECT @TmpCnt = COUNT_BIG(*) FROM #DeadlockOccurrences;
        SET @logMsg = '-- ' + CONVERT(VARCHAR(10),@TmpCnt) + ' deadlocks have to be treated';
        RAISERROR(@logMsg , 0 , 1) WITH NOWAIT;
    END;    
    
    IF(@ExecRet <> 0)
    BEGIN
        RAISERROR('Unable to get back the list of deadlocks of interest',12,1) WITH NOWAIT;
        RETURN;
    END;
    
    SELECT @CurDeadlockId = min(OccurId) 
    FROM #DeadlockOccurrences
    WHERE InError = 0
    ;
    
    WHILE(@CurDeadlockId IS NOT NULL)
    BEGIN
    
        BEGIN TRY 
            TRUNCATE TABLE #ObjectReferences;
            /*
                This code is inspired from Wayne Sheffield's solution available at:
                http://blog.waynesheffield.com/wayne/code-library/deadlocks/shred-deadlock-graph/
            */
            SET @tsql = 'WITH DeadlockDataSource AS  ('  + @LineFeed +
                        '    SELECT  '  + @LineFeed +
                        '    ' + @DeadlockIdentifierColumnName + ' AS DeadlockId,' +  @LineFeed +
                        '    ' + @DeadlockGraphColumnName + ' AS DeadlockGraph' + @LineFeed +
                        '    FROM ' + @SourceTableFullName  + @LineFeed +
                        '    WHERE ' + @DeadlockIdentifierColumnName + ' = @DeadlockId'  + @LineFeed +
                        '), '  + @LineFeed +
                        'VictimIds '  + @LineFeed +
                        'AS ('  + @LineFeed +
                        '    SELECT    Victims.List.value(''@id'', ''VARCHAR(64)'') as ID'  + @LineFeed +
                        '    FROM      DeadlockDataSource'  + @LineFeed +
                        '    CROSS APPLY        '  + @LineFeed +
                        '        DeadlockDataSource.DeadlockGraph.nodes(''//deadlock/victim-list/victimProcess'') AS Victims (List)'  + @LineFeed +
                        '), '  + @LineFeed +
                        'Locks AS ('  + @LineFeed +
                        '    -- Merge all of the lock information together.'  + @LineFeed +
                        '    SELECT  '  + @LineFeed +
                        '        DeadlockDataSource.DeadlockID,'  + @LineFeed +
                        '        MainLock.Process.value(''@id'', ''varchar(100)'') AS LockID,'  + @LineFeed +
                        '        OwnerList.Owner.value(''@id'', ''varchar(200)'') AS LockProcessId,'  + @LineFeed +
                        '        REPLACE(MainLock.Process.value(''local-name(.)'', ''varchar(128)''), ''lock'', '''') AS LockEvent,'  + @LineFeed +
                        '        MainLock.Process.value(''@objectname'', ''sysname'') AS ObjectName,'  + @LineFeed +
                        '        OwnerList.Owner.value(''@mode'', ''varchar(10)'') AS LockMode,'  + @LineFeed +
                        '        MainLock.Process.value(''@dbid'', ''INTEGER'') AS DatabaseId,'  + @LineFeed +
                        '        MainLock.Process.value(''@associatedObjectId'', ''BIGINT'') AS AssociatedObjectId,'  + @LineFeed +
                        '        MainLock.Process.value(''@WaitType'', ''varchar(100)'') AS WaitType,'  + @LineFeed +
                        '        WaitersList.Owner.value(''@id'', ''varchar(200)'') AS WaitProcessID,'  + @LineFeed +
                        '        WaitersList.Owner.value(''@mode'', ''varchar(10)'') AS WaitMode'  + @LineFeed +
                        '    FROM    '  + @LineFeed +
                        '        DeadlockDataSource'  + @LineFeed +
                        '    CROSS APPLY '  + @LineFeed +
                        '        DeadlockDataSource.DeadlockGraph.nodes(''//deadlock/resource-list'') AS Lock (list)'  + @LineFeed +
                        '    CROSS APPLY '  + @LineFeed +
                        '        Lock.list.nodes(''*'') AS MainLock (Process)'  + @LineFeed +
                        '    OUTER APPLY '  + @LineFeed +
                        '        MainLock.Process.nodes(''owner-list/owner'') AS OwnerList (Owner)'  + @LineFeed +
                        '    CROSS APPLY '  + @LineFeed +
                        '        MainLock.Process.nodes(''waiter-list/waiter'') AS WaitersList (Owner)'  + @LineFeed +
                        '), '  + @LineFeed +
                        'Processes AS  ('  + @LineFeed +
                        '    -- get data from the process node'  + @LineFeed +
                        '    SELECT  DeadlockDataSource.DeadlockID,'  + @LineFeed +
                        '            [isVictim] = CONVERT(BIT, CASE WHEN Deadlock.Process.value(''@id'', ''VARCHAR(64)'') = ISNULL(Deadlock.Process.value(''../../@victim'', ''varchar(50)''), v.ID) '  + @LineFeed +
                        '                                     THEN 1'  + @LineFeed +
                        '                                     ELSE 0'  + @LineFeed +
                        '                                END),'  + @LineFeed +
                        '            [LockMode] = Deadlock.Process.value(''@lockMode'', ''varchar(10)''), -- how is this different from in the resource-list section?'  + @LineFeed +
                        '            [ProcessID] = Process.ID, --Deadlock.Process.value(''@id'', ''VARCHAR(64)''),'  + @LineFeed +
                        '            [ThreadId] = Deadlock.Process.value(''@kpid'', ''int''), -- kernel-process id / thread ID number'  + @LineFeed +
                        '            [SPID] = Deadlock.Process.value(''@spid'', ''int''), -- system process id (connection to sql)'  + @LineFeed +
                        '            [SBID] = Deadlock.Process.value(''@sbid'', ''int''), -- system batch id / request_id (a query that a SPID is running)'  + @LineFeed +
                        '            [ECID] = Deadlock.Process.value(''@ecid'', ''int''), -- execution context ID (a worker thread running part of a query)'  + @LineFeed +
                        '            [IsolationLevel] = Deadlock.Process.value(''@isolationlevel'', ''varchar(200)''),'  + @LineFeed +
                        '            [WaitResource] = Deadlock.Process.value(''@waitresource'', ''varchar(200)''),'  + @LineFeed +
                        '            [LogUsed] = Deadlock.Process.value(''@logused'', ''int''),'  + @LineFeed +
                        '            [ClientApp] = Deadlock.Process.value(''@clientapp'', ''varchar(100)''),'  + @LineFeed +
                        '            [HostName] = Deadlock.Process.value(''@hostname'', ''varchar(20)''),'  + @LineFeed +
                        '            [LoginName] = Deadlock.Process.value(''@loginname'', ''varchar(20)''),'  + @LineFeed +
                        '            [TransactionTime] = Deadlock.Process.value(''@lasttranstarted'', ''datetime''),'  + @LineFeed +
                        '            [BatchStarted] = Deadlock.Process.value(''@lastbatchstarted'', ''datetime''),'  + @LineFeed +
                        '            [BatchCompleted] = Deadlock.Process.value(''@lastbatchcompleted'', ''datetime''),'  + @LineFeed +
                        '            [InputBuffer] = CAST(Input.Buffer.query(''.'') AS xml),'  + @LineFeed +
                        '            -- DeadlockDataSource.[DeadlockGraph],'  + @LineFeed +
                        '            es.ExecutionStack,'  + @LineFeed +
                        '            [SQLHandle] = ExecStack.Stack.value(''@sqlhandle'', ''varchar(64)''),'  + @LineFeed +
                        '            [QueryStatement] = NULLIF(ExecStack.Stack.value(''.'', ''varchar(max)''), ''''),'  + @LineFeed +
                        '            --[QueryStatement] = Execution.Frame.value(''.'', ''varchar(max)''),'  + @LineFeed +
                        '            [ProcessQty] = SUM(1) OVER (PARTITION BY DeadlockDataSource.DeadlockID),'  + @LineFeed +
                        '            [TranCount] = Deadlock.Process.value(''@trancount'', ''int'')'  + @LineFeed +
                        '    FROM    DeadlockDataSource'  + @LineFeed +
                        '        CROSS APPLY DeadlockDataSource.DeadlockGraph.nodes(''//deadlock/process-list/process'') AS Deadlock (Process)'  + @LineFeed +
                        '        CROSS APPLY (SELECT Deadlock.Process.value(''@id'', ''VARCHAR(64)'') ) AS Process (ID)'  + @LineFeed +
                        '        LEFT JOIN VictimIds v ON Process.ID = v.ID'  + @LineFeed +
                        '        CROSS APPLY Deadlock.Process.nodes(''inputbuf'') AS Input (Buffer)'  + @LineFeed +
                        '        CROSS APPLY Deadlock.Process.nodes(''executionStack'') AS Execution (Frame)'  + @LineFeed +
                        '        -- get the data from the executionStack node as XML'  + @LineFeed +
                        '        CROSS APPLY (SELECT ExecutionStack = (SELECT   ProcNumber = ROW_NUMBER() '  + @LineFeed +
                        '                                                                    OVER (PARTITION BY DeadlockDataSource.DeadlockID,'  + @LineFeed +
                        '                                                                                       Deadlock.Process.value(''@id'', ''VARCHAR(64)''),'  + @LineFeed +
                        '                                                                                       Execution.Stack.value(''@procname'', ''sysname''),'  + @LineFeed +
                        '                                                                                       Execution.Stack.value(''@code'', ''varchar(MAX)'') '  + @LineFeed +
                        '                                                                              ORDER BY (SELECT 1)),'  + @LineFeed +
                        '                                                        ProcName = Execution.Stack.value(''@procname'', ''sysname''),'  + @LineFeed +
                        '                                                        Line = Execution.Stack.value(''@line'', ''int''),'  + @LineFeed +
                        '                                                        SQLHandle = Execution.Stack.value(''@sqlhandle'', ''varchar(64)''),'  + @LineFeed +
                        '                                                        Code = LTRIM(RTRIM(Execution.Stack.value(''.'', ''varchar(MAX)'')))'  + @LineFeed +
                        '                                                FROM Execution.Frame.nodes(''frame'') AS Execution (Stack)'  + @LineFeed +
                        '                                                ORDER BY ProcNumber'  + @LineFeed +
                        '                                                FOR XML PATH(''frame''), ROOT(''executionStack''), TYPE )'  + @LineFeed +
                        '                    ) es'  + @LineFeed +
                        '        CROSS APPLY Execution.Frame.nodes(''frame'') AS ExecStack (Stack)'  + @LineFeed +
                        ')'  + @LineFeed +
                        'INSERT INTO ' + @TargetTableFullName + '(' + @LineFeed +
                        '    DeadlockID,'  + @LineFeed +
                        '    isVictim, '  + @LineFeed +
                        '    ProcessQty,'  + @LineFeed +
                        '    ProcessNbr,'  + @LineFeed +
                        '    LockMode,'  + @LineFeed +
                        '    LockedObject,'  + @LineFeed +
                        '    DatabaseId,'  + @LineFeed +
                        '    AssociatedObjectId,'  + @LineFeed +
                        '    LockProcess,'  + @LineFeed +
                        '    ThreadId,'  + @LineFeed +
                        '    SPID,'  + @LineFeed +
                        '    SBID,'  + @LineFeed +
                        '    ECID,'  + @LineFeed +
                        '    TranCount,'  + @LineFeed +
                        '    LockEvent,'  + @LineFeed +
                        '    LockedMode,'  + @LineFeed +
                        '    WaitProcessID,'  + @LineFeed +
                        '    WaitMode,'  + @LineFeed +
                        '    WaitResource,'  + @LineFeed +
                        '    WaitType,'  + @LineFeed +
                        '    IsolationLevel,'  + @LineFeed +
                        '    LogUsed,'  + @LineFeed +
                        '    ClientApp,'  + @LineFeed +
                        '    HostName,'  + @LineFeed +
                        '    LoginName,'  + @LineFeed +
                        '    TransactionTime,'  + @LineFeed +
                        '    BatchStarted,'  + @LineFeed +
                        '    BatchCompleted,'  + @LineFeed +
                        '    QueryStatement,'  + @LineFeed +
                        '    SQLHandle,'  + @LineFeed +
                        '    InputBuffer,'  + @LineFeed +
                        '    ExecutionStack'  + @LineFeed +
                        ')' + @LineFeed +
                        'SELECT  '  + @LineFeed +
                        '    p.DeadlockID,'  + @LineFeed +
                        '    p.isVictim,'  + @LineFeed +
                        '    p.ProcessQty,'  + @LineFeed +
                        '    ProcessNbr = DENSE_RANK() '  + @LineFeed +
                        '                    OVER (PARTITION BY p.DeadlockId '  + @LineFeed +
                        '                            ORDER BY p.ProcessID),'  + @LineFeed +
                        '    p.LockMode,'  + @LineFeed +
                        '    LockedObject = NULLIF(l.ObjectName, ''''),'  + @LineFeed +
                        '    l.DatabaseId,'  + @LineFeed +
                        '    l.AssociatedObjectId,'  + @LineFeed +
                        '    LockProcess = p.ProcessID,'  + @LineFeed +
                        '    p.ThreadId,'  + @LineFeed +
                        '    p.SPID,'  + @LineFeed +
                        '    p.SBID,'  + @LineFeed +
                        '    p.ECID,'  + @LineFeed +
                        '    p.TranCount,'  + @LineFeed +
                        '    l.LockEvent,'  + @LineFeed +
                        '    LockedMode = l.LockMode,'  + @LineFeed +
                        '    l.WaitProcessID,'  + @LineFeed +
                        '    l.WaitMode,'  + @LineFeed +
                        '    p.WaitResource,'  + @LineFeed +
                        '    l.WaitType,'  + @LineFeed +
                        '    p.IsolationLevel,'  + @LineFeed +
                        '    p.LogUsed,'  + @LineFeed +
                        '    p.ClientApp,'  + @LineFeed +
                        '    p.HostName,'  + @LineFeed +
                        '    p.LoginName,'  + @LineFeed +
                        '    p.TransactionTime,'  + @LineFeed +
                        '    p.BatchStarted,'  + @LineFeed +
                        '    p.BatchCompleted,'  + @LineFeed +
                        '    p.QueryStatement,'  + @LineFeed +
                        '    p.SQLHandle,'  + @LineFeed +
                        '    p.InputBuffer,'  + @LineFeed +
                        '    -- p.DeadlockGraph,'  + @LineFeed +
                        '    p.ExecutionStack'  + @LineFeed +
                        'FROM    Processes p'  + @LineFeed +
                        '        LEFT JOIN Locks l'  + @LineFeed +
                        '        --JOIN Processes p'  + @LineFeed +
                        '            ON p.DeadlockID = l.DeadlockID'  + @LineFeed +
                        '               AND p.ProcessID = l.LockProcessID'  + @LineFeed +
                        'ORDER BY p.DeadlockId,'  + @LineFeed +
                        '        p.isVictim DESC,'  + @LineFeed +
                        '        p.ProcessId'  + @LineFeed
                        ;
            
            
            EXEC @ExecRet = sp_executesql @tsql, N'@DeadlockId BIGINT', @DeadlockId = @CurDeadlockId;
            
            IF(@ExecRet <> 0)
            BEGIN
                RAISERROR('-- An error occurred during shredding',12,1) WITH NOWAIT;
            END;
            
            IF(@Debug = 1)
            BEGIN
                RAISERROR('-- Getting back actual object name',0,1) WITH NOWAIT;
            END;
            
            SET @tsql = 'INSERT INTO #ObjectReferences' + @LineFeed +
                        'SELECT distinct DatabaseId,AssociatedObjectId' + @LineFeed +
                        'FROM ' + @TargetTableFullName + @LineFeed +
                        'WHERE DeadlockId = @DeadlockId AND DatabaseId IS NOT NULL'
                        ;
                        
            EXEC @ExecRet = sp_executesql @tsql, N'@DeadlockId BIGINT', @DeadlockId = @CurDeadlockId;
            
            SELECT TOP 1
                @ObjectLookupDbId = DatabaseId,
                @ObjectLookupObjId = PartitionId
            FROM #ObjectReferences
            ;
            
            WHILE @ObjectLookupDbId IS NOT NULL AND @ObjectLookupObjId IS NOT NULL
            BEGIN
                
                SET @tsql = 'USE ' + QUOTENAME(DB_NAME(@ObjectLookupDbId)) + @LineFeed +
                            'SELECT @retVal = QUOTENAME(DB_NAME()) + ''.'' + QUOTENAME(OBJECT_SCHEMA_NAME(object_id)) + ''.'' + QUOTENAME(OBJECT_NAME(object_id))'+ @LineFeed +
                            'FROM sys.partitions' + @LineFeed +
                            'WHERE hobt_id = @partId'
                            ;
                            
                EXEC @ExecRet = sp_executesql @tsql, N'@retVal VARCHAR(1024) OUTPUT,@partId BIGINT', @retVal = @ObjectLookupName OUTPUT, @partId = @ObjectLookupObjId;
                
                IF(@ObjectLookupName IS NOT NULL AND LEN(@ObjectLookupName) > 0)
                BEGIN
                    SET @tsql = 'UPDATE ' + @TargetTableFullName + @LineFeed +
                                'SET' + @LineFeed +
                                '    AssociatedObjectName = @ObjName' + @LineFeed +
                                'WHERE ' + @LineFeed +
                                '    DeadlockId = @Did' + @LineFeed +
                                'AND DatabaseId = @DbId' + @LineFeed +
                                'AND AssociatedObjectId = @ObjId'
                    ;
                    
                    EXEC @ExecRet = sp_executesql   @tsql, 
                                                    N'@ObjName VARCHAR(1024), @Did BIGINT, @DbId INT, @ObjId BIGINT', 
                                                    @ObjName    = @ObjectLookupName, 
                                                    @Did        = @CurDeadlockId,
                                                    @DbId       = @ObjectLookupDbId,
                                                    @ObjId      = @ObjectLookupObjId
                    ;
                    
                END;
                
                
                -- carry on
                
                DELETE FROM #ObjectReferences 
                WHERE DatabaseId = @ObjectLookupDbId
                AND PartitionId = @ObjectLookupObjId
                ;
                
                SET @ObjectLookupDbId = NULL;
                
                SELECT TOP 1
                    @ObjectLookupDbId = DatabaseId,
                    @ObjectLookupObjId = PartitionId
                FROM #ObjectReferences
                ;
            END;
            
            
            IF(@Debug = 1)
            BEGIN
                RAISERROR('--Getting back SQL Text for SQL Handles',0,1) WITH NOWAIT;
            END;
            
            SET @tsql = 'WITH AllHandlesWithText' + @LineFeed +
                        'AS ('  + @LineFeed + 
                        '    SELECT sdh.DeadlockId, sdh.SQLHandle,SqlTxt.[text] as SqlText'  + @LineFeed+
                        '    FROM ' + @TargetTableFullName + ' as sdh'  + @LineFeed+
                        '    CROSS APPLY '  + @LineFeed+
                        '        sys.dm_exec_sql_text(CONVERT(VARBINARY(64),sdh.SQLHandle,1)) SqlTxt'  + @LineFeed +
                        '    WHERE sdh.DeadlockId = @DeadlockId' + @LineFeed +
                        ')'  + @LineFeed+
                        'UPDATE dh '  + @LineFeed+
                        'SET '  + @LineFeed +
                        '    dh.SQLHandleText = alh.SqlText'  + @LineFeed+
                        'FROM ' + @TargetTableFullName + ' dh'  + @LineFeed+
                        'INNER JOIN AllHandlesWithText alh'  + @LineFeed +
                        'ON  dh.DeadlockId = alh.DeadlockId' + @LineFeed +  
                        'AND dh.SQLHandle = alh.SQLHandle '  + @LineFeed 
                        ;
            
            EXEC @ExecRet = sp_executesql @tsql, N'@DeadlockId BIGINT', @DeadlockId = @CurDeadlockId;
            
            IF(@Debug = 1)
            BEGIN
                RAISERROR('-- Prepare for next iteration',0,1) WITH NOWAIT;
            END;
            
            DELETE FROM #DeadlockOccurrences WHERE OccurId = @CurDeadlockId;
        END TRY
        BEGIN CATCH
            RAISERROR('TODO: trap error and insert into #DeadlockOccurrences',12,1);
            SELECT 
                ERROR_NUMBER() AS ErrorNumber,
                ERROR_SEVERITY() AS ErrorSeverity,
                ERROR_STATE() AS ErrorState,
                ERROR_PROCEDURE() AS ErrorProcedure,
                ERROR_LINE() AS ErrorLine,
                ERROR_MESSAGE() AS ErrorMessage
            ;
            
            UPDATE #DeadlockOccurrences
            SET 
                InError = 1
            WHERE OccurId = @CurDeadlockId
            ;
        END CATCH
        
        SET @CurDeadlockId = NULL;
        
        SELECT @CurDeadlockId = min(OccurId) 
        FROM #DeadlockOccurrences
        WHERE InError = 0
        ;
    END;
    
    
    
    if(@Debug = 1)
    BEGIN
        RAISERROR('Performing cleanups.',0,1);
    END;

    IF(EXISTS(SELECT 1 FROM #DeadlockOccurrences))
    BEGIN   
        SELECT * FROM #DeadlockOccurrences ;
    END;
    
    IF(OBJECT_ID('tempdb..#DeadlockOccurrences') IS NOT NULL)
    BEGIN
        exec sp_executesql N'DROP TABLE #DeadlockOccurrences';
    END;         
    
    IF(OBJECT_ID('tempdb..#ObjectReferences') IS NOT NULL)
    BEGIN
        exec sp_executesql N'DROP TABLE #ObjectReferences';
    END;
    
    
    
                
    if (@Debug = 1)
    BEGIN
        RAISERROR('-- -----------------------------------------------------------------------------------------------------------------',0,1);
        RAISERROR('-- Execution of [Reporting].[ShredDeadlockHistoryTbl] completed.',0,1);
        RAISERROR('-- -----------------------------------------------------------------------------------------------------------------',0,1);
    END;


END
GO


IF (@@ERROR = 0)
BEGIN
    PRINT '   PROCEDURE altered.';
END
ELSE
BEGIN
    PRINT '   Error while trying to alter procedure';
    RETURN
END;
GO

RAISERROR('-----------------------------------------------------------------------------------------------------------------',0,1);
RAISERROR('',0,1);
GO

